% interior point method for linear programming

function [xstar, ystar, k, B] = ipm(c, A, b, beta, xfeas, alpha, epsilon)

  %% initialization
  OPTIONS = [];
  [m, n] = size(A);
  Ineq = A(:, 1 : n-m);
  nx = size(xfeas);
  if nx < n
    s = b - Ineq*xfeas;
    x = [ xfeas ; s ];
  else
    x = xfeas;
  end
  J = zeros(n, 1);
  H = zeros(n, n);
  d = zeros(n, 1);
  nu = zeros(m, 1);
  termination = 0;
  counter = 1;

  %% iterative method
  while termination == 0
    for i = 1 : n
      J(i) = c(i) - beta / x(i);
      H(i,i) = beta/x(i)^2;
    end
    N = [ H, A'; A, zeros(m, m) ];
    bN = [ -J; zeros(m, 1) ];
    direction = N \ bN;
    d = direction(1 : n, 1);
    nu = direction(n + 1 : n + m);
    lambda = fminbnd('linesearch', 0, 1, OPTIONS, c, x, d, beta);
    xstar = x + lambda * d;
    if n * beta < epsilon
      termination = 1;
      k = counter;
      ystar = beta ./ xstar;
      B = zeros(1, n);
      for i = 1 : n
        if xstar(i) > ystar(i)
          B(i) = 1;
        end
      end
    end
    beta = alpha * beta;
    x = xstar;
    counter = counter + 1;
  end

% end function

